﻿-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: Banzai")

local Banzai = AceLibrary("LibBanzai-2.0")
local Roster = AceLibrary("Roster-2.1")

L:RegisterTranslations("enUS", function() return {
	["Aggro duration"] = true,
	["Aggro"] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
	["Aggro duration"] = "持续被攻击目标",
	["Aggro"] = "被攻击目标(Aggro)",
} end)

L:RegisterTranslations("zhTW", function() return {
	["Aggro duration"] = "持續被攻擊目標",
	["Aggro"] = "被攻擊目標(Aggro)",
} end)

L:RegisterTranslations("koKR", function() return {
	["Aggro duration"] = "어그로 지속",
	["Aggro"] = "어그로",
} end)

L:RegisterTranslations("frFR", function() return {
	["Aggro duration"] = "Durée d'aggro",
	["Aggro"] = "Aggro",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local moduleName = "Aggro duration"
local gainedAggroTime

local PERIODICUPDATE_ID = "Violation-"..moduleName.."-PeriodicUpdate"

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 38177 $"):sub(12, -3)) or 1
mod.version = 5
mod.violationCategory = "other"

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

local function callback(aggro, name, ...)
	if aggro == 1 then
		mod:GainedAggro(name)
	elseif aggro == 0 then
		mod:LostAggro(name)
	end
end

function mod:OnModuleEnable()
	Banzai:RegisterCallback(callback)
	self:SetupAggroTable()
end

function mod:OnModuleDisable()
	self:ClearAggroTable()
	self:CancelAllScheduledEvents()
	Banzai:UnregisterCallback(callback)
end

function mod:OnReset()
	gainedAggroTime = mod:del(gainedAggroTime)
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetDisplayName() return L["Aggro duration"] end
function mod:GetShortDisplayName() return L["Aggro"] end

function mod:IsDirty()
	return self.dirty or (gainedAggroTime and next(gainedAggroTime))
end

function mod:IsBroadcastDirty( charName )
	if not charName then
		charName = self.playerName
	end
	return self.broadcast[charName] or (gainedAggroTime and gainedAggroTime[charName])
end

function mod:ClearAggroTable()
	if gainedAggroTime then
		for name in pairs(gainedAggroTime) do
			self:LostAggro(name)
		end
		gainedAggroTime = mod:del(gainedAggroTime)
	end
end

function mod:SetupAggroTable()
	gainedAggroTime = mod:del(gainedAggroTime) -- we never know
	gainedAggroTime = mod:new()
	for unit in Roster:IterateRoster(true) do
		if Banzai:GetUnitAggroByUnitName(unit.name) then
			self:GainedAggro(unit.name)
		end
	end
end

function mod:UpdateData(name)
	if not gainedAggroTime[name] then return end
	local now = GetTime()
	local delta = now - gainedAggroTime[name]
	gainedAggroTime[name] = now
	self:OnData(name, delta)
end

function mod:LostAggro(name)
	if self:AcceptDataFor(name) and gainedAggroTime and gainedAggroTime[name] then
		self:UpdateData(name)
		gainedAggroTime[name] = nil
	end
end

function mod:GainedAggro(name)
	if self:AcceptDataFor(name) and (not gainedAggroTime or not gainedAggroTime[name]) then
		if not gainedAggroTime then
			gainedAggroTime = self:new()
		end
		gainedAggroTime[name] = GetTime()
		self:UpdateData(name)
	end
	if gainedAggroTime and next(gainedAggroTime) and not self:IsEventScheduled(PERIODICUPDATE_ID) then
		self:ScheduleRepeatingEvent(PERIODICUPDATE_ID, self.PeriodicUpdate, 1, self) 
	end
end

function mod:PeriodicUpdate()
	if gainedAggroTime then
		for name in pairs(gainedAggroTime) do
			self:UpdateData(name)
		end
	end
	if gainedAggroTime and not next(gainedAggroTime) and self:IsEventScheduled(PERIODICUPDATE_ID) then
		self:CancelScheduledEvent(PERIODICUPDATE_ID) 
	end
end

local floor = math.floor
function mod.dataFormatter(duration)
	if duration >= 3600 then
		return ("%d:%02d:%02d"):format(floor(duration / 3600), floor(duration/60) % 60, floor(duration) % 60)
	else
		return ("%d:%02d"):format(floor(duration / 60), floor(duration) % 60)
	end
end

